//+------------------------------------------------------------------+
//|    MTF    Dynamic Fibonacci Grid (DFG basic v1.2)                |                                                      
//+------------------------------------------------------------------+

#property indicator_chart_window

extern int  Grid_Range  = 30;
extern int  Shift       = -98;
extern int  Spacing     = 25;
extern int  Width       = 24;
extern int  MA_mode     = 0;
extern double  gap      = 10.1;

//---------------------------------

#define NFL    10       // number of fibo levels
#define NSVR   17       // number of solid vertical lines
#define NMA    7        // number of MAs, change ma_period[NMA-1] accordingly 
#define NTF    4        // number of time frames/lanes, change tfp[NTF] accordinlgy

double   fma[NTF][NMA];                                    // fibo MAs
double   fib_up[NTF][NFL];                                 // fib levels to the up side
double   fib_down[NTF][NFL];                               // fib levels to the down side
double   fib[NFL];                                         // fib distances ('point' adjusted)
double   srl[28];                                          // support/resistance levels (total = 2*NFL + NMA + 1(Bid) )
double   price, point;
double   tmpArray[NMA], lowestMA, highestMA;
int      _fib[NFL]    = {10, 16, 26, 42, 68, 111, 179, 290, 470, 760}; // fib levels in pips : 10, 16, 26, 42, 68, 111, 179, 290, 470, 760
int      ma_period[7] = {13,21,34,55,89,144,233};         // SMAs, NMA-1
int      tfp[NTF]     = {1,5,15,60};                       // Time frames/lanes - in minutes. Like H4=240, D1=1440 etc...
int      bar, i;
int      tf, c, offset, xUnit;
int      shift, spacing, width, supports, resistances;
color    Shade[NFL] = {C'200,240,255', C'160,220,255', C'120,200,255', C'80,180,255', C'40,140,255', C'20,100,255', C'0,60,240', C'0,40,220', C'0,20,200'};
color    col, macolor;
string   objName, MA, name;
string   uniqueid = "FXG: ";
datetime curTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{          
   xUnit   = Period() * 60;
   shift   = Shift * xUnit;
   spacing = Spacing * xUnit;
   width   = Width * xUnit;
    
   point   = 0.01;
   if(Digits > 3) point = 0.0001;
    
   ChartSetInteger(0,CHART_AUTOSCROLL,true);
   ChartSetInteger(0,CHART_SHIFT,false);
   ChartSetInteger(0,CHART_MODE,CHART_LINE);
   ChartSetInteger(0,CHART_SHOW_GRID,false);
   ChartSetInteger(0,CHART_SHOW_PERIOD_SEP,false);  
   ChartSetInteger(0,CHART_SHOW_OHLC,false);  
   ChartSetInteger(0,CHART_SHOW_ASK_LINE,false); 
   ChartSetInteger(0,CHART_SHOW_OBJECT_DESCR,false);     
   ChartSetInteger(0,CHART_SHOW_VOLUMES,CHART_VOLUME_HIDE);
   ChartSetInteger(0,CHART_FOREGROUND,false);   
         
   ChartSetInteger(0,CHART_COLOR_CHART_LINE,clrNONE);
   ChartSetInteger(0,CHART_COLOR_BACKGROUND,clrDarkGray);
   ChartSetInteger(0,CHART_COLOR_FOREGROUND,clrDarkGray);
   ChartSetInteger(0,CHART_COLOR_GRID,clrBlack);     
   ChartSetInteger(0,CHART_COLOR_CANDLE_BULL,clrNONE);
   ChartSetInteger(0,CHART_COLOR_CANDLE_BEAR,clrNONE);      
   ChartSetInteger(0,CHART_COLOR_CHART_UP,clrNONE);
   ChartSetInteger(0,CHART_COLOR_CHART_DOWN,clrNONE);  
   ChartSetInteger(0,CHART_COLOR_VOLUME,clrGreen);  
   ChartSetInteger(0,CHART_COLOR_STOP_LEVEL,clrRed);  
            
   ChartSetInteger(0,CHART_SCALEFIX,true);
   ChartSetInteger(0,CHART_SHOW_DATE_SCALE,false);
   ChartSetDouble(0,CHART_FIXED_MAX,Bid+(Grid_Range*point));
   ChartSetDouble(0,CHART_FIXED_MIN,Bid-(Grid_Range*point));
   
//+------------------------------------------------------------------+
//| creating the objects for drawing the grid                        |
//+------------------------------------------------------------------+

   for(i=0; i<NFL; i++)  fib[i] = _fib[i]*point;
   for(tf=0; tf<NTF; tf++)
   {
      for(i=0; i<NFL; i++)
      {
         objName = uniqueid + "fib_up_rect_" + tf + "_" + i;      // upper zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, Shade[i]);
            
         objName = uniqueid + "ext_fib_up_rect_" + tf;            // external upper zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, C'255,255,255');
            
         objName = uniqueid + "fib_down_rect_" + tf + "_" + i;    // lower zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, Shade[i]);
            
         objName = uniqueid + "ext_fib_down_rect_" + tf;          // external lower zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, C'255,255,255');
            
         objName = uniqueid + "fib_up_level_"+tf+"_"+i;           // upper fib levels
         ObjectCreate(objName, OBJ_TEXT, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, clrBlack);
            
         objName = uniqueid + "fib_down_level_"+tf+"_"+i;         // lower fib levels
         ObjectCreate(objName, OBJ_TEXT, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, clrBlack);
      }

         objName = uniqueid + "MidShade_" + tf;                   // price zone
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, C'255,255,255');

      for(i=0; i<NMA; i++)
      {
         objName = uniqueid + "fma_" + tf + "_" + i;              // fibo MAs levels
         ObjectCreate(objName, OBJ_TEXT, 0, 0, 0, 0, 0);
         ObjectSetInteger(NULL,objName, OBJPROP_ANCHOR,ANCHOR_CENTER);
         ObjectSet(objName, OBJPROP_COLOR, Black);
         ObjectSetInteger(NULL,objName,OBJPROP_FONTSIZE,7);
      }

      for(i=1; i<=NSVR; i++)
      {
         objName = uniqueid + "svr_" + tf + "_" + i;              // solid vertical lines (10 pips or more)
         ObjectCreate(objName, OBJ_TREND, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_RAY, false);
         ObjectSet(objName, OBJPROP_WIDTH, 3);
            
         objName = uniqueid + "ext_up_svr_" + tf;                 // extern upper vertical lines
         ObjectCreate(objName, OBJ_TREND, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_RAY, false);
         ObjectSet(objName, OBJPROP_WIDTH, 3);
            
         objName = uniqueid + "ext_down_svr_" + tf;               // extern lower vertical lines
         ObjectCreate(objName, OBJ_TREND, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_RAY, false);
         ObjectSet(objName, OBJPROP_WIDTH, 3);
      }
   }
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+

int deinit()
{
   ObjectDelete("Bid");  
      
   string lookFor = uniqueid;
   int lookForLength = StringLen(lookFor);   
   for(int x = ObjectsTotal() - 1; x >= 0; x--)
      {
      string xname = ObjectName(x);
      if(StringSubstr(xname, 0, lookForLength) == lookFor) ObjectDelete(xname);
      }
      
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   point   = 0.01;
   if(Digits > 3) point = 0.0001;
   
   ChartSetInteger(0,CHART_SHOW_DATE_SCALE,false);
   ChartSetDouble(0,CHART_FIXED_MAX,Bid+(Grid_Range*point));
   ChartSetDouble(0,CHART_FIXED_MIN,Bid-(Grid_Range*point));
   
      ObjectDelete("Bid");
   ObjectCreate(0,"Bid",OBJ_HLINE,0,0,Bid);
   ObjectSet("Bid",OBJPROP_STYLE,5);
   ObjectSet("Bid",OBJPROP_COLOR,clrBlack);
   ObjectSet("Bid",OBJPROP_SELECTABLE,false);

//+------------------------------------------------------------------+
//| Time lines                                                       |
//+------------------------------------------------------------------+

   int short_gap_count;
   int long_gap_count;
   int sr_short_gap_count;
   int sr_long_gap_count;
   
   curTime = Time[0];
   price   = Bid;
    
   for(tf=0; tf<NTF; tf++)
   {
      bar = iBarShift(NULL, tfp[tf], curTime, false);
      for(i=0; i<NMA; i++)  fma[tf][i] = iMA(NULL, tfp[tf], ma_period[i], 0, MA_mode, PRICE_CLOSE, bar/Period());  // populating the fma array...
      for(i=0; i<NMA; i++) tmpArray[i] = fma[tf][i];                                                               // finding the highest and lowest MAs from the (7) fma's/MA's
      ArraySort(tmpArray);                                                                                         // copying the MAs levels of this TF to one dim array to feed ArrayBsearch
         lowestMA  = tmpArray[0];
         highestMA = tmpArray[6];

      for(i=0; i<NFL; i++)                                                                                         // populating the fib arrays...
      {
         fib_up[tf][i]   = lowestMA  + fib[i];
         fib_down[tf][i] = highestMA - fib[i];
      }
   }

//+------------------------------------------------------------------+
//| drawing the color shades of the grid from top to bottom          |
//+------------------------------------------------------------------+

   for(tf=0; tf<NTF; tf++) 
   {
      offset = 0;                                                                // find the number of overlapping shades
      for(i=offset; i<NFL-1; i++) if(fib_up[tf][i] <= fib_down[tf][i]) offset++;

      if(offset > 0)                                                             // hide first level objects' overlapping, so they wont be left as 'ghosts'
      {
         objName = uniqueid + "fib_up_rect_" + tf + "_" + (NFL-2-offset+1);
         ObjectSet(objName, OBJPROP_TIME1,  0);
         ObjectSet(objName, OBJPROP_TIME2,  0);
         ObjectSet(objName, OBJPROP_PRICE1, 0);
         ObjectSet(objName, OBJPROP_PRICE2, 0);
            
         objName = uniqueid + "fib_down_rect_" + tf + "_" + (NFL-2-offset+1);
         ObjectSet(objName, OBJPROP_TIME1,  0);
         ObjectSet(objName, OBJPROP_TIME2,  0);
         ObjectSet(objName, OBJPROP_PRICE1, 0);
         ObjectSet(objName, OBJPROP_PRICE2, 0);

      }
        
      double highest=0;
      for(i=NFL-2; i>=offset; i--)
      {
         objName = uniqueid + "fib_up_rect_" + tf + "_" + (i-offset);
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, fib_up[tf][i]);
         ObjectSet(objName, OBJPROP_PRICE2, fib_up[tf][i+1]);
                
         if (fib_down[tf][i]>highest) highest=fib_up[tf][i+1];
            
         objName = uniqueid + "fib_up_level_"+tf+"_"+(i-offset);
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_PRICE1, fib_up[tf][i]);
         ObjectSetInteger(0,objName,OBJPROP_ANCHOR,ANCHOR_CENTER);
         ObjectSetInteger(NULL,objName, OBJPROP_FONTSIZE,8);
         ObjectSetString(NULL,objName, OBJPROP_FONT, "Arial");
            
         string upper = DoubleToString(fib_up[tf][i],Digits);
         int SLength1 = StringLen(upper);
         string upperL = StringSubstr(upper, SLength1-3, 2);
            
         ObjectSetText(objName,  "-"+ upperL + "-");           
      }

      double lowest=9999.9999;
      for(i=0; i<NFL-1; i++)
      {
         objName = uniqueid + "fib_down_rect_" + tf + "_" + (i-offset);
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, fib_down[tf][i]);
         ObjectSet(objName, OBJPROP_PRICE2, fib_down[tf][i+1]);
         
         if(fib_down[tf][i]<lowest) lowest=fib_down[tf][i+1];
            
         objName = uniqueid + "fib_down_level_"+tf+"_"+(i-offset);
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_PRICE1, fib_down[tf][i]);
         ObjectSetInteger(NULL,objName,OBJPROP_ANCHOR,ANCHOR_CENTER);
         ObjectSetInteger(NULL,objName, OBJPROP_FONTSIZE,8);
         ObjectSetString(NULL,objName, OBJPROP_FONT, "Arial");
            
         string lower = DoubleToString(fib_down[tf][i],Digits);
         int SLength2 = StringLen(lower);
         string lowerL = StringSubstr(lower, SLength2-3, 2);

         ObjectSetText(objName,  "-"+ lowerL + "-");
      }

         objName = uniqueid + "MidShade_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, fib_up[tf][offset]);
         ObjectSet(objName, OBJPROP_PRICE2, fib_down[tf][offset]);
      
         objName = uniqueid + "ext_fib_up_rect_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, highest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMax());
            
         objName = uniqueid + "ext_up_svr_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_PRICE1, highest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMax());   
         ObjectSet(objName, OBJPROP_COLOR,  clrNONE);
      
         objName = uniqueid + "ext_fib_down_rect_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, lowest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMin());
        
         objName = uniqueid + "ext_down_svr_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+Width/2*xUnit);
         ObjectSet(objName, OBJPROP_PRICE1, lowest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMin());
         ObjectSet(objName, OBJPROP_COLOR,  clrNONE);

//+------------------------------------------------------------------+
//| drawing the SMA price levels and "boxes"                         |
//+------------------------------------------------------------------+

      for(i=0; i<NMA; i++)
      {
         objName = uniqueid + "fma_" + tf + "_" + i;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+6*xUnit);
         ObjectSet(objName, OBJPROP_PRICE1, fma[tf][i]);
            
         if(i==0 || i==1)
         {            
            if(i==0)if(fma[tf][i] >= price) col = clrRed;
            else col = clrLime;
            
            if(i==1)if(fma[tf][i] >= price) col = clrFireBrick;
            else col = clrForestGreen;
            
            ObjectSet(objName, OBJPROP_COLOR, col);
            ObjectSetInteger(NULL,objName, OBJPROP_FONTSIZE,9);
            ObjectSetString(NULL,objName, OBJPROP_FONT, "Marlett");
            ObjectSetText(objName,"g");
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+3*xUnit);

            ObjectCreate(NULL,objName+"_",OBJ_TEXT,0,0,0,0);
            ObjectSet(objName+"_", OBJPROP_COLOR, Black);
            ObjectSetInteger(NULL,objName+"_",OBJPROP_ANCHOR,ANCHOR_LEFT);
            ObjectSetInteger(NULL,objName+"_", OBJPROP_FONTSIZE,7);
            ObjectSetString(NULL,objName+"_", OBJPROP_FONT, "Arial");

            string labelString0 = DoubleToString(fma[tf][i],Digits);
            int SLength0 = StringLen(labelString0);
            string maString0 = StringSubstr(labelString0, SLength0-3, 2);
                        
            ObjectSet(objName+"_", OBJPROP_TIME1,  curTime+shift+tf*spacing+5*xUnit);
            ObjectSet(objName+"_", OBJPROP_PRICE1, fma[tf][i]);
            ObjectSetText(objName+"_", " " + maString0);            
         }

         if(i>1 && i<6)
         {
            ObjectSetInteger(NULL,objName,OBJPROP_ANCHOR,ANCHOR_LEFT);
            ObjectSetString(NULL,objName, OBJPROP_FONT, "Arial");
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+5*xUnit);
            
            string labelStringX = DoubleToString(fma[tf][i],Digits);
            int SLengthX = StringLen(labelStringX);
            string maStringX = StringSubstr(labelStringX, SLengthX-3, 2);            
            ObjectSetText(objName, " " + maStringX);
         }
         
         
         if(i==6)
         {
            col = Black;
            ObjectSet(objName, OBJPROP_COLOR, col);
            ObjectSetInteger(NULL,objName, OBJPROP_FONTSIZE,9);
            ObjectSetString(NULL,objName, OBJPROP_FONT, "Marlett");
            ObjectSetText(objName,"g");
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing+3*xUnit);

            ObjectCreate(NULL,objName+"_",OBJ_TEXT,0,0,0,0);
            ObjectSet(objName+"_", OBJPROP_COLOR, Black);
            ObjectSetInteger(NULL,objName+"_",OBJPROP_ANCHOR,ANCHOR_LEFT);
            ObjectSetInteger(NULL,objName+"_", OBJPROP_FONTSIZE,7);
            ObjectSetString(NULL,objName+"_", OBJPROP_FONT, "Arial");

            string labelString6 = DoubleToString(fma[tf][i],Digits);
            int SLength6 = StringLen(labelString6);
            string maString6 = StringSubstr(labelString6, SLength6-3, 2);
            
            ObjectSet(objName+"_", OBJPROP_TIME1,  curTime+shift+tf*spacing+5*xUnit);
            ObjectSet(objName+"_", OBJPROP_PRICE1, fma[tf][i]);
            ObjectSetText(objName+"_", " " + maString6);            
         }
      }

//+------------------------------------------------------------------+
//| drawing the solid vertical lines                                 |
//+------------------------------------------------------------------+      

      ArrayInitialize(srl, -1);
      for (i=0; i<NMA; i++) srl[i] = fma[tf][i];
      for (i=offset; i<NFL; i++)
      {
         srl[NMA+i] = fib_down[tf][i];
         srl[NMA+NFL+i] = fib_up[tf][i];
      }
      
      srl[NMA+2*NFL] = price;
      ArraySort(srl);

      c           = 0;        // looking for a gap of 10 pips or more
      supports    = 0;
      resistances = 0;
      for(i=0; i<NMA+2*NFL+2; i++)
      {
         if(srl[i] > price) resistances++;
         if(srl[i] < price) supports++;
         if(-1 == srl[i])
            continue;
         if(srl[i+1]-srl[i] >= gap*point)
         {
            c++;
            objName = uniqueid + "svr_" + tf + "_" + c;
               ObjectSet(objName, OBJPROP_TIME1, curTime+shift+tf*spacing+Width/2*xUnit);
               ObjectSet(objName, OBJPROP_TIME2, curTime+shift+tf*spacing+Width/2*xUnit);
               ObjectSet(objName, OBJPROP_PRICE1, srl[i]+ Point()*11);
               ObjectSet(objName, OBJPROP_PRICE2, srl[i+1]-Point()*10);

            if (srl[i+1] <= price)
            {
               if(price==srl[i+1]) short_gap_count++;
               ObjectSet(objName, OBJPROP_COLOR, FireBrick);
            }
            else
            {
               if(price==srl[i]) long_gap_count++;
               ObjectSet(objName, OBJPROP_COLOR, Green);
            }
         }
         
         if (srl[i+1]-srl[i] >= gap*point)
         {
            if(srl[i+1] <= price)
               if(price==srl[i+1]) sr_short_gap_count++;
               else
               if(price==srl[i])   sr_long_gap_count++;
         }

      }
      
      if(supports == 0) 
      {
         sr_short_gap_count++;
         short_gap_count++;
      }
      
      if(resistances == 0) 
      {
         sr_long_gap_count++;
         long_gap_count++;
      }

      if(c >= NSVR) Print("Warning: NSVR exceeded");
      else  // hide the unused vertical lines objects so they wont be left as 'ghosts' (only the first)
      {
         objName = uniqueid + "svr_" + tf + "_" + (c+1);
            ObjectSet(objName, OBJPROP_TIME1, 0);
            ObjectSet(objName, OBJPROP_TIME2, 0);
            ObjectSet(objName, OBJPROP_PRICE1, 0);
            ObjectSet(objName, OBJPROP_PRICE2, 0);
      }
   }
   
//--------------------------------------------------------------------------
return(0);
}

